@@ -15,110 +15,6 @@ wavelet_test (OilTest *test)
   }
 }
 
-static void
-mas_test (OilTest *test)
-{
-  int16_t *data;
-  int i;
-  int n;
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC1);
-  for(i=0;i<test->n;i++){
-    //data[i] = oil_rand_s16()>>1;
-    data[i] = 0;
-  }
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC2);
-  for(i=0;i<test->n;i++){
-    data[i] = oil_rand_s16();
-  }
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC3);
-  n = oil_test_get_arg_post_n (test, OIL_ARG_SRC3);
-  for(i=0;i<n;i++){
-    data[i] = (oil_rand_s16()>>4)/n;
-  }
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC4);
-  data[0] = (1<<11);
-  data[1] = 12;
-}
-
-static void
-mas2_across_test (OilTest *test)
-{
-  int16_t *data;
-  int i;
-  int n;
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC1);
-  for(i=0;i<test->n;i++){
-    //data[i] = oil_rand_s16()>>1;
-    data[i] = 0;
-  }
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC2);
-  for(i=0;i<test->n;i++){
-    data[i] = oil_rand_s16()>>4;
-  }
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC3);
-  for(i=0;i<test->n;i++){
-    data[i] = oil_rand_s16()>>4;
-  }
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC4);
-  n = oil_test_get_arg_post_n (test, OIL_ARG_SRC4);
-  for(i=0;i<n;i++){
-    data[i] = (oil_rand_s16()>>4)/n;
-  }
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC5);
-  data[0] = (1<<11);
-  data[1] = 12;
-}
-
-static void
-mas48_across_test (OilTest *test)
-{
-  int16_t *data;
-  int stride;
-  int i;
-  int j;
-  int n;
-  int m;
-  int sum;
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC1);
-  for(i=0;i<test->n;i++){
-    //data[i] = oil_rand_s16()>>1;
-    data[i] = 0;
-  }
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC2);
-  m = oil_test_get_arg_post_n (test, OIL_ARG_SRC2);
-  stride = oil_test_get_arg_stride (test, OIL_ARG_SRC2);
-  for(j=0;j<m;j++){
-    for(i=0;i<test->n;i++){
-      data[i] = oil_rand_s16()>>12;
-    }
-    data = OIL_OFFSET(data, stride);
-  }
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC3);
-  n = oil_test_get_arg_post_n (test, OIL_ARG_SRC3);
-  sum = 0;
-  for(i=0;i<n-1;i++){
-    data[i] = (oil_rand_s16()>>8)/n;
-    sum += data[i];
-  }
-  data[i] = 128 - sum;
-
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC4);
-  data[0] = (1<<6);
-  data[1] = 7;
-}
-
 static void
 rshift_test (OilTest *test)
 {
@@ -143,7 +39,59 @@ lshift_test (OilTest *test)
   data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC2);
   data[0] = 12;
 }
+ 
+static void
+combine2_test (OilTest *test)
+{
+  int16_t *data;
 
+  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC3);
+  data[0] = 1;
+  data[1] = 1;
+  data[2] = 1;
+  data[3] = 1;
+}
+
+static void
+combine4_test (OilTest *test)
+{
+  int16_t *data;
+
+  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC5);
+  data[0] = 4;
+  data[1] = 4;
+  data[2] = 4;
+  data[3] = 4;
+  data[4] = 8;
+  data[5] = 4;
+}
+
+static void
+add2_test (OilTest *test)
+{
+  int16_t *data;
+  int i;
+
+  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC1);
+  for(i=0;i<test->n;i++){
+    data[i] = oil_rand_s16()>>4;
+  }
+
+  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC2);
+  for(i=0;i<test->n;i++){
+    data[i] = oil_rand_s16()>>4;
+  }
+
+  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC3);
+  for(i=0;i<test->n;i++){
+    data[i] = oil_rand_s16()>>4;
+  }
+
+  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC4);
+  data[0] = 1;
+  data[1] = 1;
+}
+ 
 OIL_DEFINE_CLASS_FULL (deinterleave,
     "int16_t *d_2xn, int16_t *s_2xn, int n", wavelet_test);
 OIL_DEFINE_CLASS (deinterleave2_s16,
@@ -168,34 +116,49 @@ OIL_DEFINE_CLASS_FULL (split_135,
     "int16_t *d_2xn, int16_t *s_2xn, int n", wavelet_test);
 OIL_DEFINE_CLASS_FULL (synth_135,
     "int16_t *d_2xn, int16_t *s_2xn, int n", wavelet_test);
-OIL_DEFINE_CLASS_FULL (mas2_add_s16,
-    "int16_t *d, int16_t *s1, int16_t *s2_np1, int16_t *s3_2, int16_t *s4_2, "
-    "int n", mas_test);
-OIL_DEFINE_CLASS_FULL (mas4_add_s16,
-    "int16_t *d, int16_t *s1, int16_t *s2_np3, int16_t *s3_4, int16_t *s4_2, "
-    "int n", mas_test);
-OIL_DEFINE_CLASS_FULL (mas8_add_s16,
-    "int16_t *d, int16_t *s1, int16_t *s2_np7, int16_t *s3_8, int16_t *s4_2, "
-    "int n", mas_test);
 OIL_DEFINE_CLASS_FULL (add_const_rshift_s16,
     "int16_t *d1, int16_t *s1, int16_t *s2_2, int n", rshift_test);
 OIL_DEFINE_CLASS_FULL (lshift_s16,
     "int16_t *d1, int16_t *s1, int16_t *s2_1, int n", lshift_test);
-OIL_DEFINE_CLASS_FULL (mas2_across_add_s16,
-    "int16_t *d, int16_t *s1, int16_t *s2, int16_t *s3, int16_t *s4_2, "
-    "int16_t *s5_2, int n", mas2_across_test);
-OIL_DEFINE_CLASS_FULL (mas4_across_add_s16,
-    "int16_t *d, int16_t *s1, int16_t *s2_nx4, int sstr2, int16_t *s3_4, "
-    "int16_t *s4_2, int n", mas48_across_test);
-OIL_DEFINE_CLASS_FULL (mas8_across_add_s16,
-    "int16_t *d, int16_t *s1, int16_t *s2_nx8, int sstr2, int16_t *s3_8, "
-    "int16_t *s4_2, int n", mas48_across_test);
 OIL_DEFINE_CLASS (multiply_and_add_s16,
     "int16_t *d, int16_t *src1, int16_t *src2, int16_t *src3, int n");
 OIL_DEFINE_CLASS (multiply_and_add_s16_u8,
     "int16_t *d, int16_t *src1, int16_t *src2, uint8_t *src3, int n");
+OIL_DEFINE_CLASS (multiply_and_acc_6xn_s16_u8, "int16_t *i1_6xn, int is1, "
+    "int16_t *s1_6xn, int ss1, uint8_t *s2_6xn, int ss2, int n");
+OIL_DEFINE_CLASS (multiply_and_acc_8xn_s16_u8, "int16_t *i1_8xn, int is1, "
+    "int16_t *s1_8xn, int ss1, uint8_t *s2_8xn, int ss2, int n");
 OIL_DEFINE_CLASS (multiply_and_acc_12xn_s16_u8, "int16_t *i1_12xn, int is1, "
     "int16_t *s1_12xn, int ss1, uint8_t *s2_12xn, int ss2, int n");
+OIL_DEFINE_CLASS (multiply_and_acc_16xn_s16_u8, "int16_t *i1_16xn, int is1, "
+    "int16_t *s1_16xn, int ss1, uint8_t *s2_16xn, int ss2, int n");
+OIL_DEFINE_CLASS (multiply_and_acc_24xn_s16_u8, "int16_t *i1_24xn, int is1, "
+    "int16_t *s1_24xn, int ss1, uint8_t *s2_24xn, int ss2, int n");
+OIL_DEFINE_CLASS_FULL (combine2_8xn_u8, "uint8_t *d_8xn, int ds1, "
+    "uint8_t *s1_8xn, int ss1, uint8_t *s2_8xn, int ss2, int16_t *s3_4, int n", combine2_test);
+OIL_DEFINE_CLASS_FULL (combine2_12xn_u8, "uint8_t *d_12xn, int ds1, "
+    "uint8_t *s1_12xn, int ss1, uint8_t *s2_12xn, int ss2, int16_t *s3_4, int n", combine2_test);
+OIL_DEFINE_CLASS_FULL (combine2_16xn_u8, "uint8_t *d_16xn, int ds1, "
+    "uint8_t *s1_16xn, int ss1, uint8_t *s2_16xn, int ss2, int16_t *s3_4, int n", combine2_test);
+OIL_DEFINE_CLASS_FULL (combine4_8xn_u8, "uint8_t *d_8xn, int ds1, "
+    "uint8_t *s1_8xn, int ss1, uint8_t *s2_8xn, int ss2, uint8_t *s3_8xn, "
+    "int ss3, uint8_t *s4_8xn, int ss4, int16_t *s5_6, int n", combine4_test);
+OIL_DEFINE_CLASS_FULL (combine4_12xn_u8, "uint8_t *d_12xn, int ds1, "
+    "uint8_t *s1_12xn, int ss1, uint8_t *s2_12xn, int ss2, uint8_t *s3_12xn, "
+    "int ss3, uint8_t *s4_12xn, int ss4, int16_t *s5_6, int n", combine4_test);
+OIL_DEFINE_CLASS_FULL (combine4_16xn_u8, "uint8_t *d_16xn, int ds1, "
+    "uint8_t *s1_16xn, int ss1, uint8_t *s2_16xn, int ss2, uint8_t *s3_16xn, "
+    "int ss3, uint8_t *s4_16xn, int ss4, int16_t *s5_6, int n", combine4_test);
+OIL_DEFINE_CLASS_FULL (add2_rshift_add_s16, "int16_t *d, int16_t *s1, "
+    "int16_t *s2, int16_t *s3, int16_t *s4_2, int n", add2_test);
+OIL_DEFINE_CLASS_FULL (add2_rshift_sub_s16, "int16_t *d, int16_t *s1, "
+    "int16_t *s2, int16_t *s3, int16_t *s4_2, int n", add2_test);
+OIL_DEFINE_CLASS (avg2_8xn_u8, "uint8_t *d_8xn, int ds1, "
+    "uint8_t *s1_8xn, int ss1, uint8_t *s2_8xn, int ss2, int n");
+OIL_DEFINE_CLASS (avg2_12xn_u8, "uint8_t *d_12xn, int ds1, "
+    "uint8_t *s1_12xn, int ss1, uint8_t *s2_12xn, int ss2, int n");
+OIL_DEFINE_CLASS (avg2_16xn_u8, "uint8_t *d_16xn, int ds1, "
+    "uint8_t *s1_16xn, int ss1, uint8_t *s2_16xn, int ss2, int n");
 
 void
 deinterleave_ref (int16_t *d_2xn, int16_t *s_2xn, int n)
@@ -514,13 +477,6 @@ lift_test (OilTest *test)
   for(i=0;i<test->n;i++){
     data[i] = oil_rand_u8();
   }
-#if 0
-  data = (int16_t *)oil_test_get_source_data (test, OIL_ARG_SRC4);
-  if (data) {
-    data[0] = oil_rand_u8();
-    data[1] = oil_rand_u8();
-  }
-#endif
 }
 
 OIL_DEFINE_CLASS_FULL (lift_add_shift1,
@@ -624,56 +580,6 @@ lift_sub_135_ref (int16_t *d, int16_t *s1, int16_t *s2, int16_t *s3,
 OIL_DEFINE_IMPL_REF (lift_sub_135_ref, lift_sub_135);
 
 
-void
-mas2_add_s16_ref(int16_t *d1, int16_t *s1, int16_t *s2, int16_t *s3_2,
-    int16_t *s4_2, int n)
-{
-  int i;
-  int x;
-
-  for(i=0;i<n;i++){
-    x = s4_2[0] + s2[i]*s3_2[0] + s2[i+1]*s3_2[1];
-    x >>= s4_2[1];
-    d1[i] = s1[i] + x;
-  }
-}
-OIL_DEFINE_IMPL_REF (mas2_add_s16_ref, mas2_add_s16);
-
-void
-mas4_add_s16_ref(int16_t *d1, int16_t *s1, int16_t *s2, int16_t *s3_4,
-    int16_t *s4_2, int n)
-{
-  int i;
-  int x;
-
-  for(i=0;i<n;i++){
-    x = s4_2[0] + s2[i]*s3_4[0] + s2[i+1]*s3_4[1] + s2[i+2]*s3_4[2] +
-        s2[i+3]*s3_4[3];
-    x >>= s4_2[1];
-    d1[i] = s1[i] + x;
-  }
-}
-OIL_DEFINE_IMPL_REF (mas4_add_s16_ref, mas4_add_s16);
-
-void
-mas8_add_s16_ref(int16_t *d1, int16_t *s1, int16_t *s2, int16_t *s3_8,
-    int16_t *s4_2, int n)
-{
-  int i;
-  int j;
-  int x;
-
-  for(i=0;i<n;i++){
-    x = s4_2[0];
-    for(j=0;j<8;j++){
-      x += s2[i+j]*s3_8[j];
-    }
-    x >>= s4_2[1];
-    d1[i] = s1[i] + x;
-  }
-}
-OIL_DEFINE_IMPL_REF (mas8_add_s16_ref, mas8_add_s16);
-
 void
 add_const_rshift_s16_ref(int16_t *d1, int16_t *s1, int16_t *s3_2, int n)
 {
@@ -694,57 +600,6 @@ lshift_s16_ref(int16_t *d1, int16_t *s1, int16_t *s3_1, int n)
 }
 OIL_DEFINE_IMPL_REF (lshift_s16_ref, lshift_s16);
 
-void
-mas2_across_add_s16_ref (int16_t *d, int16_t *s1, int16_t *s2, int16_t *s3,
-    int16_t *s4_2, int16_t *s5_2, int n)
-{
-  int i;
-  int x;
-  for(i=0;i<n;i++){
-    x = s5_2[0];
-    x += s2[i]*s4_2[0] + s3[i]*s4_2[1];
-    x >>= s5_2[1];
-    d[i] = s1[i] + x;
-  }
-}
-OIL_DEFINE_IMPL_REF (mas2_across_add_s16_ref, mas2_across_add_s16);
-
-void
-mas4_across_add_s16_ref (int16_t *d, int16_t *s1, int16_t *s2_nx4, int sstr2,
-    int16_t *s3_4, int16_t *s4_2, int n)
-{
-  int i;
-  int j;
-  int x;
-  for(i=0;i<n;i++){
-    x = s4_2[0];
-    for(j=0;j<4;j++){
-      x += OIL_GET(s2_nx4, i*sizeof(int16_t) + j*sstr2, int16_t)*s3_4[j];
-    }
-    x >>= s4_2[1];
-    d[i] = s1[i] + x;
-  }
-}
-OIL_DEFINE_IMPL_REF (mas4_across_add_s16_ref, mas4_across_add_s16);
-
-void
-mas8_across_add_s16_ref (int16_t *d, int16_t *s1, int16_t *s2_nx8, int sstr2,
-    int16_t *s3_8, int16_t *s4_2, int n)
-{
-  int i;
-  int j;
-  int x;
-  for(i=0;i<n;i++){
-    x = s4_2[0];
-    for(j=0;j<8;j++){
-      x += OIL_GET(s2_nx8, i*sizeof(int16_t) + j*sstr2, int16_t)*s3_8[j];
-    }
-    x >>= s4_2[1];
-    d[i] = s1[i] + x;
-  }
-}
-OIL_DEFINE_IMPL_REF (mas8_across_add_s16_ref, mas8_across_add_s16);
-
 void
 multiply_and_add_s16_ref (int16_t *d, int16_t *src1, int16_t *src2, int16_t *src3, int n)
 {
@@ -766,6 +621,40 @@ multiply_and_add_s16_u8_ref (int16_t *d, int16_t *src1, int16_t *src2,
 }
 OIL_DEFINE_IMPL_REF (multiply_and_add_s16_u8_ref, multiply_and_add_s16_u8);
 
+void
+multiply_and_acc_6xn_s16_u8_ref (int16_t *i1, int is1, int16_t *s1,
+    int ss1, uint8_t *s2, int ss2, int n)
+{
+  int i, j;
+  for(j=0;j<n;j++){
+    for(i=0;i<6;i++){
+      i1[i] += s1[i]*s2[i];
+    }
+    i1 = OIL_OFFSET(i1,is1);
+    s1 = OIL_OFFSET(s1,ss1);
+    s2 = OIL_OFFSET(s2,ss2);
+  }
+}
+OIL_DEFINE_IMPL_REF (multiply_and_acc_6xn_s16_u8_ref,
+    multiply_and_acc_6xn_s16_u8);
+
+void
+multiply_and_acc_8xn_s16_u8_ref (int16_t *i1, int is1, int16_t *s1,
+    int ss1, uint8_t *s2, int ss2, int n)
+{
+  int i, j;
+  for(j=0;j<n;j++){
+    for(i=0;i<8;i++){
+      i1[i] += s1[i]*s2[i];
+    }
+    i1 = OIL_OFFSET(i1,is1);
+    s1 = OIL_OFFSET(s1,ss1);
+    s2 = OIL_OFFSET(s2,ss2);
+  }
+}
+OIL_DEFINE_IMPL_REF (multiply_and_acc_8xn_s16_u8_ref,
+    multiply_and_acc_8xn_s16_u8);
+
 void
 multiply_and_acc_12xn_s16_u8_ref (int16_t *i1, int is1, int16_t *s1,
     int ss1, uint8_t *s2, int ss2, int n)
@@ -783,3 +672,260 @@ multiply_and_acc_12xn_s16_u8_ref (int16_t *i1, int is1, int16_t *s1,
 OIL_DEFINE_IMPL_REF (multiply_and_acc_12xn_s16_u8_ref,
     multiply_and_acc_12xn_s16_u8);
 
+void
+multiply_and_acc_16xn_s16_u8_ref (int16_t *i1, int is1, int16_t *s1,
+    int ss1, uint8_t *s2, int ss2, int n)
+{
+  int i, j;
+  for(j=0;j<n;j++){
+    for(i=0;i<16;i++){
+      i1[i] += s1[i]*s2[i];
+    }
+    i1 = OIL_OFFSET(i1,is1);
+    s1 = OIL_OFFSET(s1,ss1);
+    s2 = OIL_OFFSET(s2,ss2);
+  }
+}
+OIL_DEFINE_IMPL_REF (multiply_and_acc_16xn_s16_u8_ref,
+    multiply_and_acc_16xn_s16_u8);
+
+void
+multiply_and_acc_24xn_s16_u8_ref (int16_t *i1, int is1, int16_t *s1,
+    int ss1, uint8_t *s2, int ss2, int n)
+{
+  int i, j;
+  for(j=0;j<n;j++){
+    for(i=0;i<24;i++){
+      i1[i] += s1[i]*s2[i];
+    }
+    i1 = OIL_OFFSET(i1,is1);
+    s1 = OIL_OFFSET(s1,ss1);
+    s2 = OIL_OFFSET(s2,ss2);
+  }
+}
+OIL_DEFINE_IMPL_REF (multiply_and_acc_24xn_s16_u8_ref,
+    multiply_and_acc_24xn_s16_u8);
+
+void
+combine4_8xn_u8_ref (uint8_t *d, int ds1,
+    uint8_t *s1, int ss1,
+    uint8_t *s2, int ss2,
+    uint8_t *s3, int ss3,
+    uint8_t *s4, int ss4,
+    int16_t *s5_6, int n)
+{
+  int i;
+  int j;
+  for(j=0;j<n;j++){
+    for(i=0;i<8;i++){
+      int x = 0;
+      x += s5_6[0] * s1[i];
+      x += s5_6[1] * s2[i];
+      x += s5_6[2] * s3[i];
+      x += s5_6[3] * s4[i];
+      d[i] = (x + s5_6[4]) >> s5_6[5];
+    }
+    s1 += ss1;
+    s2 += ss2;
+    s3 += ss3;
+    s4 += ss4;
+    d += ds1;
+  }
+}
+OIL_DEFINE_IMPL_REF (combine4_8xn_u8_ref, combine4_8xn_u8);
+
+void
+combine4_12xn_u8_ref (uint8_t *d, int ds1,
+    uint8_t *s1, int ss1,
+    uint8_t *s2, int ss2,
+    uint8_t *s3, int ss3,
+    uint8_t *s4, int ss4,
+    int16_t *s5_6, int n)
+{
+  int i;
+  int j;
+  for(j=0;j<n;j++){
+    for(i=0;i<12;i++){
+      int x = 0;
+      x += s5_6[0] * s1[i];
+      x += s5_6[1] * s2[i];
+      x += s5_6[2] * s3[i];
+      x += s5_6[3] * s4[i];
+      d[i] = (x + s5_6[4]) >> s5_6[5];
+    }
+    s1 += ss1;
+    s2 += ss2;
+    s3 += ss3;
+    s4 += ss4;
+    d += ds1;
+  }
+}
+OIL_DEFINE_IMPL_REF (combine4_12xn_u8_ref, combine4_12xn_u8);
+
+void
+combine4_16xn_u8_ref (uint8_t *d, int ds1,
+    uint8_t *s1, int ss1,
+    uint8_t *s2, int ss2,
+    uint8_t *s3, int ss3,
+    uint8_t *s4, int ss4,
+    int16_t *s5_6, int n)
+{
+  int i;
+  int j;
+  for(j=0;j<n;j++){
+    for(i=0;i<16;i++){
+      int x = 0;
+      x += s5_6[0] * s1[i];
+      x += s5_6[1] * s2[i];
+      x += s5_6[2] * s3[i];
+      x += s5_6[3] * s4[i];
+      d[i] = (x + s5_6[4]) >> s5_6[5];
+    }
+    s1 += ss1;
+    s2 += ss2;
+    s3 += ss3;
+    s4 += ss4;
+    d += ds1;
+  }
+}
+OIL_DEFINE_IMPL_REF (combine4_16xn_u8_ref, combine4_16xn_u8);
+
+void
+combine2_8xn_u8_ref (uint8_t *d, int ds1,
+    uint8_t *s1, int ss1,
+    uint8_t *s2, int ss2,
+    int16_t *s3_4, int n)
+{
+  int i;
+  int j;
+  for(j=0;j<n;j++){
+    for(i=0;i<8;i++){
+      int x = 0;
+      x += s3_4[0] * s1[i];
+      x += s3_4[1] * s2[i];
+      d[i] = (x + s3_4[2]) >> s3_4[3];
+    }
+    s1 += ss1;
+    s2 += ss2;
+    d += ds1;
+  }
+}
+OIL_DEFINE_IMPL_REF (combine2_8xn_u8_ref, combine2_8xn_u8);
+
+void
+combine2_12xn_u8_ref (uint8_t *d, int ds1,
+    uint8_t *s1, int ss1,
+    uint8_t *s2, int ss2,
+    int16_t *s3_4, int n)
+{
+  int i;
+  int j;
+  for(j=0;j<n;j++){
+    for(i=0;i<12;i++){
+      int x = 0;
+      x += s3_4[0] * s1[i];
+      x += s3_4[1] * s2[i];
+      d[i] = (x + s3_4[2]) >> s3_4[3];
+    }
+    s1 += ss1;
+    s2 += ss2;
+    d += ds1;
+  }
+}
+OIL_DEFINE_IMPL_REF (combine2_12xn_u8_ref, combine2_12xn_u8);
+
+void
+combine2_16xn_u8_ref (uint8_t *d, int ds1,
+    uint8_t *s1, int ss1,
+    uint8_t *s2, int ss2,
+    int16_t *s3_4, int n)
+{
+  int i;
+  int j;
+  for(j=0;j<n;j++){
+    for(i=0;i<16;i++){
+      int x = 0;
+      x += s3_4[0] * s1[i];
+      x += s3_4[1] * s2[i];
+      d[i] = (x + s3_4[2]) >> s3_4[3];
+    }
+    s1 += ss1;
+    s2 += ss2;
+    d += ds1;
+  }
+}
+OIL_DEFINE_IMPL_REF (combine2_16xn_u8_ref, combine2_16xn_u8);
+
+void
+add2_rshift_add_s16_ref (int16_t *d, int16_t *s1, int16_t *s2, int16_t *s3,
+    int16_t *s4_2, int n)
+{
+  int i;
+  for(i=0;i<n;i++) {
+    d[i] = s1[i] + ((s2[i] + s3[i] + s4_2[0])>>s4_2[1]);
+  }
+}
+OIL_DEFINE_IMPL_REF (add2_rshift_add_s16_ref, add2_rshift_add_s16);
+
+void
+add2_rshift_sub_s16_ref (int16_t *d, int16_t *s1, int16_t *s2, int16_t *s3,
+    int16_t *s4_2, int n)
+{
+  int i;
+  for(i=0;i<n;i++) {
+    d[i] = s1[i] - ((s2[i] + s3[i] + s4_2[0])>>s4_2[1]);
+  }
+}
+OIL_DEFINE_IMPL_REF (add2_rshift_sub_s16_ref, add2_rshift_sub_s16);
+
+void
+avg2_8xn_u8_ref (uint8_t *d, int ds1, uint8_t *s1, int ss1,
+    uint8_t *s2, int ss2, int n)
+{
+  int i;
+  int j;
+  for(j=0;j<n;j++){
+    for(i=0;i<8;i++){
+      d[i] = (s1[i] + s2[i] + 1)>>1;
+    }
+    s1 += ss1;
+    s2 += ss2;
+    d += ds1;
+  }
+}
+OIL_DEFINE_IMPL_REF (avg2_8xn_u8_ref, avg2_8xn_u8);
+
+void
+avg2_12xn_u8_ref (uint8_t *d, int ds1, uint8_t *s1, int ss1,
+    uint8_t *s2, int ss2, int n)
+{
+  int i;
+  int j;
+  for(j=0;j<n;j++){
+    for(i=0;i<12;i++){
+      d[i] = (s1[i] + s2[i] + 1)>>1;
+    }
+    s1 += ss1;
+    s2 += ss2;
+    d += ds1;
+  }
+}
+OIL_DEFINE_IMPL_REF (avg2_12xn_u8_ref, avg2_12xn_u8);
+
+void
+avg2_16xn_u8_ref (uint8_t *d, int ds1, uint8_t *s1, int ss1,
+    uint8_t *s2, int ss2, int n)
+{
+  int i;
+  int j;
+  for(j=0;j<n;j++){
+    for(i=0;i<16;i++){
+      d[i] = (s1[i] + s2[i] + 1)>>1;
+    }
+    s1 += ss1;
+    s2 += ss2;
+    d += ds1;
+  }
+}
+OIL_DEFINE_IMPL_REF (avg2_16xn_u8_ref, avg2_16xn_u8);
+
