@@ -32,7 +32,9 @@
 #include <emmintrin.h>
 #include <xmmintrin.h>
 
-static void
+#define SSE_FUNCTION __attribute__((force_align_arg_pointer))
+
+SSE_FUNCTION static void
 add_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 {
   /* Initial operations to align the destination pointer */
@@ -59,7 +61,7 @@ add_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 }
 OIL_DEFINE_IMPL_FULL (add_f32_sse_unroll2, add_f32, OIL_IMPL_FLAG_SSE);
 
-static void
+SSE_FUNCTION static void
 subtract_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 {
   /* Initial operations to align the destination pointer */
@@ -86,7 +88,7 @@ subtract_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 }
 OIL_DEFINE_IMPL_FULL (subtract_f32_sse_unroll2, subtract_f32, OIL_IMPL_FLAG_SSE);
 
-static void
+SSE_FUNCTION static void
 multiply_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 {
   /* Initial operations to align the destination pointer */
@@ -113,7 +115,7 @@ multiply_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 }
 OIL_DEFINE_IMPL_FULL (multiply_f32_sse_unroll2, multiply_f32, OIL_IMPL_FLAG_SSE);
 
-static void
+SSE_FUNCTION static void
 divide_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 {
   /* Initial operations to align the destination pointer */
@@ -140,7 +142,7 @@ divide_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 }
 OIL_DEFINE_IMPL_FULL (divide_f32_sse_unroll2, divide_f32, OIL_IMPL_FLAG_SSE);
 
-static void
+SSE_FUNCTION static void
 minimum_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 {
   /* Initial operations to align the destination pointer */
@@ -171,7 +173,7 @@ minimum_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 }
 OIL_DEFINE_IMPL_FULL (minimum_f32_sse_unroll2, minimum_f32, OIL_IMPL_FLAG_SSE);
 
-static void
+SSE_FUNCTION static void
 maximum_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 {
   /* Initial operations to align the destination pointer */
@@ -202,7 +204,7 @@ maximum_f32_sse_unroll2 (float *dest, float *src1, float *src2, int n)
 }
 OIL_DEFINE_IMPL_FULL (maximum_f32_sse_unroll2, maximum_f32, OIL_IMPL_FLAG_SSE);
 
-static void
+SSE_FUNCTION static void
 inverse_f32_sse_unroll2 (float *dest, float *src1, int n)
 {
   /* Initial operations to align the destination pointer */
@@ -231,7 +233,7 @@ inverse_f32_sse_unroll2 (float *dest, float *src1, int n)
 }
 OIL_DEFINE_IMPL_FULL (inverse_f32_sse_unroll2, inverse_f32, OIL_IMPL_FLAG_SSE);
 
-static void
+SSE_FUNCTION static void
 negative_f32_sse_unroll2 (float *dest, float *src1, int n)
 {
   /* Initial operations to align the destination pointer */
@@ -257,7 +259,7 @@ negative_f32_sse_unroll2 (float *dest, float *src1, int n)
 }
 OIL_DEFINE_IMPL_FULL (negative_f32_sse_unroll2, negative_f32, OIL_IMPL_FLAG_SSE);
 
-static void
+SSE_FUNCTION static void
 scalaradd_f32_ns_sse_unroll2 (float *dest, float *src1, float *val, int n)
 {
   __m128 xmm1;
@@ -284,7 +286,7 @@ scalaradd_f32_ns_sse_unroll2 (float *dest, float *src1, float *val, int n)
 }
 OIL_DEFINE_IMPL_FULL (scalaradd_f32_ns_sse_unroll2, scalaradd_f32_ns, OIL_IMPL_FLAG_SSE);
 
-static void
+SSE_FUNCTION static void
 scalarmultiply_f32_ns_sse_unroll2 (float *dest, float *src1, float *val, int n)
 {
   __m128 xmm1;
@@ -311,7 +313,7 @@ scalarmultiply_f32_ns_sse_unroll2 (float *dest, float *src1, float *val, int n)
 }
 OIL_DEFINE_IMPL_FULL (scalarmultiply_f32_ns_sse_unroll2, scalarmultiply_f32_ns, OIL_IMPL_FLAG_SSE);
 
-static void
+SSE_FUNCTION static void
 scalarmultiply_f64_ns_sse2_unroll2 (double *dest, double *src1, double *val, int n)
 {
   __m128d xmm1;
